import { nextTick, css, html, Component, classMap, outsideClick } from "wkit";
import "../icon/index.js";
const ANIMATION = {
  duration: 100,
  custom: [
    { transform: "scaleY(0)", opacity: 0 },
    { transform: "scaleY(1)", opacity: 1 }
  ]
};
class Input extends Component {
  static props = {
    readOnly: false,
    autofocus: false,
    disabled: false,
    clearable: false,
    icon: "",
    placeholder: "",
    maxlength: { type: Number, default: null },
    minlength: { type: Number, default: null },
    value: "str!",
    lazy: "num!0"
    // 并发拦截时间, 单位毫秒
  };
  #list = [];
  #selectIndex = -1;
  #listShowing = false;
  #stamp = 0;
  static styles = [
    css`:host{position:relative;display:inline-flex;min-width:188px;height:32px;user-select:none;-moz-user-select:none;color:var(--color-dark-1);border-radius:3px;cursor:text;transition:box-shadow .15s linear}.label{flex:1;display:flex;justify-content:center;align-items:center;height:100%;font-size:14px;border:1px solid var(--wc-input-border-color, var(--color-grey-2));border-radius:inherit;background:var(--bg-color, #fff);color:inherit;cursor:inherit}.label input{flex:1;min-width:36px;width:0;height:100%;padding:0 8px;border:0;border-radius:inherit;font:inherit;color:inherit;background:none;outline:none;box-shadow:none;cursor:inherit}.label input::placeholder{color:var(--color-grey-1)}.label .close{--wc-icon-size: 18px;margin:0 8px 0 4px;padding:4px;border-radius:50%;color:var(--color-grey-2);cursor:pointer;transition:background .15s linear}.label .close:hover{background:var(--color-plain-1)}.label .icon{--wc-icon-size: 16px;margin:0 8px 0 4px;color:var(--color-grey-2)}.suggestion{overflow:hidden;position:absolute;z-index:1;left:0;top:calc(100% + 4px);width:100%;padding:4px 0;border-radius:4px;box-shadow:0 2px 5px rgba(0,0,0,.15);transform-origin:top}.suggestion .list{width:100%;background:#fff}.suggestion.hide{display:none}.suggestion li{overflow:hidden;width:100%;height:30px;line-height:30px;padding:0 8px;text-overflow:ellipsis;white-space:nowrap;cursor:pointer}.suggestion li:hover,.suggestion li[focus]{background:var(--color-plain-2)}:host([round]){border-radius:26px}:host([round]) .label input{padding:0 10px}:host(:focus-within){box-shadow:0 0 0 2px var(--color-plain-a)}:host([disabled]){pointer-events:none;cursor:not-allowed}:host([disabled]) .label{border-color:var(--color-grey-1);background:var(--color-plain-1);opacity:.6}:host([readonly]){cursor:default}`,
    //尺寸
    css`:host([size=m]){min-width:128px;height:24px;font-size:12px}:host([size=m]) .label{height:24px;font-size:12px}:host([size=m]) .icon{--wc-icon-size: 12px}:host([size=m][circle]){width:24px;height:24px}:host([size=xl]){min-width:224px;height:36px;font-size:14px}:host([size=xl]) .label{height:36px;font-size:14px}:host([size=xl]) .icon{--wc-icon-size: 14px}:host([size=xl][circle]){width:36px;height:36px}:host([size=xxl]){min-width:288px;height:44px;font-size:14px}:host([size=xxl]) .label{height:44px;font-size:14px}:host([size=xxl]) .icon{--wc-icon-size: 14px}:host([size=xxl][circle]){width:44px;height:44px}`
  ];
  renderClear() {
    return html`<wc-icon class="close" name="close" @click=${this.clear} />`;
  }
  render() {
    let classes = classMap({
      suggestion: true,
      hide: !this.#list.length
    });
    return html`
      <div class="label">
        <slot name="prepend">
          <wc-icon class="icon" name=${this.icon}></wc-icon>
        </slot>
        <input
          ref="input"
          @input=${this.handleInput}
          @change=${this.handleChange}
          @keydown=${this.handleKeyDown}
          @focus=${this.handleFocus}
          placeholder=${this.placeholder}
          maxlength=${this.maxlength}
          minlength=${this.minlength}
          disabled=${this.disabled}
          readonly=${this.readOnly}
          autofocus=${this.autofocus}
          :value=${this.value}
        />
        ${this.clearable && this.value ? this.renderClear() : ""}
        <slot name="append"></slot>
        <div class=${classes} ref="suggestion" #animation=${ANIMATION}>
          <div ref="scroller" class="scroller">
            <ul class="list" @click=${this.handleClickItem} ref="list">
              ${this.#list.map(
      (li, idx) => html`<li
                    focus=${this.#selectIndex === idx ? true : null}
                    index=${idx}
                  >
                    ${li.value}
                  </li>`
    )}
            </ul>
          </div>
        </div>
      </div>
    `;
  }
  handleInput(e) {
    let { lazy } = this;
    this.value = e.currentTarget.value;
    if (lazy && Date.now() - this.#stamp < lazy) {
      return;
    }
    this.#stamp = Date.now();
    this.emitFetchSuggest();
  }
  handleClickItem(e) {
    let index = e.target.getAttribute("index");
    this.#selectIndex = index;
    this.emitSelect();
  }
  clear() {
    this.$refs.input.value = "";
    this.value = "";
    this.$emit("change");
    this.$emit("input");
  }
  handleChange() {
    this.$emit("change");
  }
  handleKeyDown(e) {
    let { lazy, minlength, value } = this;
    if (e.keyCode === 13) {
      e.preventDefault();
      if (this.#selectIndex > -1 && this.#listShowing) {
        return this.emitSelect();
      }
      if (lazy && Date.now() - this.#stamp < lazy) {
        return;
      }
      this.#stamp = Date.now();
      if (minlength && value.length < minlength) {
        return;
      }
      return this.$emit("submit");
    }
    if (e.keyCode === 38 || e.keyCode === 40) {
      e.preventDefault();
      let step = e.keyCode === 38 ? -1 : 1;
      this.#selectIndex += step;
      if (this.#selectIndex < 0) {
        this.#selectIndex = 0;
      }
      if (this.#selectIndex > this.#list.length - 1) {
        this.#selectIndex = this.#list.length - 1;
      }
      this.$requestUpdate();
    }
  }
  // 触发列表选择
  emitSelect() {
    let item = this.#list[this.#selectIndex];
    this.value = item.value;
    this.$refs.suggestion.$animate(true);
    this.#listShowing = false;
    this.$requestUpdate();
    this.$emit("change");
    this.$emit("input");
    this.$emit("select", {
      index: this.#selectIndex,
      value: item
    });
  }
  emitFetchSuggest() {
    this.$emit("fetch-suggest", {
      value: this.value,
      send: (list) => {
        this.#list = list.slice(0, 10);
        this.#selectIndex = -1;
        this.$requestUpdate();
      }
    });
  }
  handleFocus() {
    if (!this.#listShowing) {
      this.#listShowing = true;
      this.$refs.suggestion.$animate();
    }
  }
  mounted() {
    if (this.autofocus) {
      nextTick((_) => this.$refs.input.focus());
    }
    outsideClick(this, () => {
      this.#listShowing = false;
      this.$refs.suggestion.$animate(true);
    });
  }
}
Input.reg("input");
